<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\PostRequest;
use App\Models\Category;
use App\Models\Post;
use App\Models\Tag;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;

class PostController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $posts = Post::with(['user', 'category', 'tags', 'comments'])->paginate(10);

        return view('admin.posts.index', compact('posts'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories = Category::pluck('name', 'id')->all();
        $tags = Tag::pluck('name', 'name')->all();

        return view('admin.posts.create', compact('categories', 'tags'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(PostRequest $request)
    {
        request()->validate([
            'title' => 'required',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            'body'  => 'required',
            'category_id' => 'required',
        ],
        [
            'title.required' => 'Post title is required.',
            'image.required' => 'Post Image is required',
            'body.required'  => 'Post description is required',
            'category_id.required' => 'Select Categort is required.',
        ]);
        $post = new Post;
        $slug = $post->setSlugAttribute($request->title);
        $imageName = "";
        if($request->hasFile('image')){
            $imageName = time().'.'.$request->image->extension();
            $request->image->move(public_path('img'), $imageName);
        }

        $is_popular = (isset($request->is_popular)) ? 1 : 0;
        $is_trending = (isset($request->is_trending)) ? 1 : 0;

        $post = Post::create([
            'title'       => $request->title,
            'slug'        => $slug,
            'image'       => $imageName,
            'body'        => $request->body,
            'is_popular'  => $is_popular,
            'is_trending' => $is_trending,
            'category_id' => $request->category_id,
        ]);

        $tagsId = collect($request->tags)->map(function ($tag) {
            $tagC = new Tag;
            $slug = $tagC->setSlugAttribute($tag);
            return Tag::firstOrCreate(['name' => $tag,'slug' => $slug])->id;
        });

        $post->tags()->attach($tagsId);
        flash()->overlay('Post created successfully.');

        return redirect('/admin/posts');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Post $post)
    {
        $post = $post->load(['user', 'category', 'tags', 'comments']);

        return view('admin.posts.show', compact('post'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Post $post)
    {
        if ($post->user_id != auth()->user()->id && auth()->user()->is_admin == false) {
            flash()->overlay("You can't edit other peoples post.");

            return redirect('/admin/posts');
        }

        $categories = Category::pluck('name', 'id')->all();
        $tags = Tag::pluck('name', 'name')->all();

        return view('admin.posts.edit', compact('post', 'categories', 'tags'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(PostRequest $request, Post $post)
    {
        // disbust cache
        Cache::forget($post->etag);

        request()->validate([
            'title' => 'required',
            'body'  => 'required',
            'category_id' => 'required',
        ],
        [
            'title.required' => 'Post title is required.',
            'body.required'  => 'Post description is required',
            'category_id.required' => 'Select Categort is required.',
        ]);

        $is_popular = (isset($request->is_popular)) ? 1 : 0;
        $is_trending = (isset($request->is_trending)) ? 1 : 0;
    
        if($request->hasFile('image')){
            $imageName = time().'.'.$request->image->extension();
            $request->image->move(public_path('img'), $imageName);
            $post->update([
                'title'       => $request->title,
                'image'       => $imageName,
                'body'        => $request->body,
                'is_popular'  => $is_popular,
                'is_trending' => $is_trending,
                'category_id' => $request->category_id,
            ]);
        }
        else{
            $post->update([
                'title'       => $request->title,
                'body'        => $request->body,
                'is_popular'  => $is_popular,
                'is_trending' => $is_trending,
                'category_id' => $request->category_id,
            ]);
        }

        $tagsId = collect($request->tags)->map(function ($tag) {
            return Tag::firstOrCreate(['name' => $tag])->id;
        });

        $post->tags()->sync($tagsId);
        flash()->overlay('Post updated successfully.');

        return redirect('/admin/posts');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Post $post)
    {
        if ($post->user_id != auth()->user()->id && auth()->user()->is_admin == false) {
            flash()->overlay("You can't delete other peoples post.");

            return redirect('/admin/posts');
        }

        $post->delete();
        flash()->overlay('Post deleted successfully.');

        return redirect('/admin/posts');
    }

    public function publish(Post $post)
    {
        $post->is_published = ! $post->is_published;
        $post->save();
        flash()->overlay('Post changed successfully.');

        return redirect('/admin/posts');
    }
}
